/*
	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	ProWesS (simple) calculator

	(c) PROGS, Professional & Graphical Software
	written by Nathan Van der Auwera
	started 1 November 1995
	02.12.1995 - added some configurable items (item names, font & fontsize and
			   colour for calculator display (Joachim)
			   - now window only x scalable (Joachim)
			   - problem to select hex/bin/dec on keypress fixed (Joachim)
			   - extra zeros in binary fixed (Joachim)
	14.01.1996 - added HELP (Joachim)
	16.01.1996 - changed keypresses for 'Hex', 'Dec' and 'Bin' (Joachim)
	18.01.1996 - no longer posible to deselect current mode (Joachim)
	29.03.1996 - can make <enter> and <do> different (configurable) (Joachim)
			   - window is removed when ESC pressed twice or when ESC pressed
				 when the display is cleared (Joachim)
	20.04.1997 - pointer now centered in calculator display (Joachim)
*/

#include <sms.h>
#include <win.h>
#include <str.h>
#include <math.h>
#include <ProWesS.h>

/* configurable items */
extern char StartAsleep;
extern char CursorSeparate;
extern short DisplayFontSize;
extern char DisplayFont[];
extern char DisplayPaper;
extern char ItemHex[];
extern char ItemDec[];
extern char ItemBin[];
extern char ItemCl[];
extern char ManualDirectory[];

/* we need a global datastructure to know which "key" is pressed, and to store
 * the rest.
 */

#define MAX_STR_LEN 	40

typedef enum {
	Op_none,
	Op_plus,
	Op_min,
	Op_div,
	Op_mult,
	Op_equal
} Operator_t;


typedef enum {
	DM_Hex,
	DM_Dec,
	DM_Bin
} DisplayMode_t;

typedef struct {
	pt fontsize;
	/* the menu-structure */
	PWObject display;
	char disp_txt[MAX_STR_LEN+1];
	PWObject key_hex, key_dec, key_bin;
	PWObject key_a, key_b, key_c, key_d, key_e, key_f;
	PWObject key_7, key_8, key_9;
	PWObject key_4, key_5, key_6;
	PWObject		key_2, key_3;
	PWObject		key_pt, key_eq;

	/* some globals */
	DisplayMode_t dMode;
	int containsPoint;		/* if 0 - no point, else it indicates the power of
							 * ten for the next digit to add */
	double tmpRslt, typedIn;
	char isClear;

	Operator_t usedOp;
} Globals;

/** Forward Declarations **/
Error NumericKeyPressed(PWObject );
Error OperatorKeyPressed(PWObject );
Error AlfaKeyPressed(PWObject);
Error ModeKeyPressed(PWObject);
Error PointPressed(PWObject );
Error EqualPressed(PWObject );
Error ClearPressed(PWObject );
Error ESCPressed(PWObject );
Error redraw(PWObject, CanvasInfo *);

/*************************************************
 *				init
 *
 * the "main" function, which will initialise our window, and
 * hand the control over to the window manager (ProWesS).
 * Our window will bear heavy ressemblences with a calculator (we hope).
 * And it will be on purpose.
 *
 */

void init()
{
	PWObject obj,sep1;
	Globals g;

	LinkPROforma();

	/* initialisation of g */
	g.containsPoint = 0;
	g.tmpRslt = g.typedIn = 0;
	g.usedOp = Op_none;
	g.dMode = DM_Dec;
	g.isClear=TRUE;
	STRCopySafe("0",g.disp_txt,MAX_STR_LEN);

	PWCreate(NULL,&obj,PW_TYPE_OUTLINE,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_QUIT_KEYPRESS, 'q',
				PW_OUTLINE_TITLE_TEXT, "calculator",
				PW_OUTLINE_SLEEP_TEXT, "calculator",
				PW_OUTLINE_HELP,
				PW_OUTLINE_HELP_FILE, "calc_html",
				PW_OUTLINE_HELP_DIRECTORY, ManualDirectory,
				PW_GLOBAL_AUXILIARY, &g,
				PW_WINDOW_NOYSCALE,
				( StartAsleep ? PW_SYSTEM_SLEEP : PW_DUMMY ),
				PW_CURSOR_SEPARATE, CursorSeparate,
				NULL);

	if (!DisplayFontSize)
		PWQuery(obj,PW_DEFAULT_FONTSIZE,&g.fontsize);
	else
		g.fontsize=short2pt(DisplayFontSize);

	PWCreate(obj, &g.display, PW_TYPE_CANVAS,
				PW_CANVAS_YSIZE, g.fontsize*3/2,
				PW_CANVAS_XSIZE, g.fontsize*10,
				PW_CANVAS_ACTION_REDRAW, &redraw,
				PW_OBJECT_POINTER_START,
				NULL);

	PWCreate(obj,&sep1,PW_TYPE_SEPARATOR,
				NULL);

	PWCreate(obj,&g.key_hex,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemHex,
				PW_LOOSE_KEYPRESS, 'x',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, ModeKeyPressed,
				NULL);
	PWCreate(obj,&g.key_dec,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemDec,
				PW_LOOSE_KEYPRESS, 't',
				PW_LOOSE_ACTION_HIT, ModeKeyPressed,
				PW_LOOSE_STATUS, PW_STATUS_SELECTED,
				NULL);
	PWCreate(obj,&g.key_bin,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemBin,
				PW_LOOSE_KEYPRESS, 'i',
				PW_LOOSE_ACTION_HIT, ModeKeyPressed,
				NULL);
	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemCl,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, ClearPressed,
				NULL);
	PWCreate(obj,NULL,PW_TYPE_KEYPRESS,
				PW_KEYPRESS, 27,
				PW_KEYPRESS_ACTION, ESCPressed,
				NULL);

	PWCreate(obj,&g.key_a,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "A",
				PW_LOOSE_KEYPRESS, 'a',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	PWCreate(obj,&g.key_b,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "B",
				PW_LOOSE_KEYPRESS, 'b',
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	PWCreate(obj,&g.key_c,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "C",
				PW_LOOSE_KEYPRESS, 'c',
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	PWCreate(obj,&g.key_d,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "D",
				PW_LOOSE_KEYPRESS, 'd',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	PWCreate(obj,&g.key_e,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "E",
				PW_LOOSE_KEYPRESS, 'e',
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	PWCreate(obj,&g.key_f,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "F",
				PW_LOOSE_KEYPRESS, 'f',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, AlfaKeyPressed,
				PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE,
				NULL);
	

	PWCreate(obj,&g.key_7,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "7",
				PW_LOOSE_KEYPRESS, '7',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_8,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "8",
				PW_LOOSE_KEYPRESS, '8',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_9,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "9",
				PW_LOOSE_KEYPRESS, '9',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "+",
				PW_LOOSE_KEYPRESS, '+',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, OperatorKeyPressed,
				NULL);

	PWCreate(obj,&g.key_4,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "4",
				PW_LOOSE_KEYPRESS, '4',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_5,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "5",
				PW_LOOSE_KEYPRESS, '5',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_6,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "6",
				PW_LOOSE_KEYPRESS, '6',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "-",
				PW_LOOSE_KEYPRESS, '-',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, OperatorKeyPressed,
				NULL);



	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "1",
				PW_LOOSE_KEYPRESS, '1',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_2,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "2",
				PW_LOOSE_KEYPRESS, '2',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_3,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "3",
				PW_LOOSE_KEYPRESS, '3',
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "*",
				PW_LOOSE_KEYPRESS, '*',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, OperatorKeyPressed,
				NULL);


	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "0",
				PW_LOOSE_KEYPRESS, '0',
				PW_POSITION_NEXT_ROW,
				PW_LOOSE_ACTION_HIT, NumericKeyPressed,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				NULL);
	PWCreate(obj,&g.key_pt,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ".",
				PW_LOOSE_KEYPRESS, '.',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, PointPressed,
				NULL);
	PWCreate(obj,&g.key_eq,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "=",
				PW_LOOSE_KEYPRESS, '=',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, EqualPressed,
				NULL);

	/* make sure ENTER and DO have the same effect as pressing '=' */
	PWCreate(g.key_eq, NULL, PW_TYPE_KEYPRESS,
				PW_KEYPRESS, 10,
				NULL);
	PWCreate(g.key_eq, NULL, PW_TYPE_KEYPRESS,
				PW_KEYPRESS, KEYSTROKE_DO,
				NULL);

	PWCreate(obj,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "/",
				PW_LOOSE_KEYPRESS, '/',
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, OperatorKeyPressed,
				NULL);


	PWActivate(obj);
}



/*****************************
 *
 * here follow the functions for the display
 */

Error redraw(PWObject obj, CanvasInfo *rinfo)
{
	Globals *g;
	int i,j,siz;
	pt width;
	Gstate gstate=rinfo->gstate;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);

	/* clear the canvas */
	PFPaperColourGray(gstate, DisplayPaper ? pt_hundred : 0 );
	PFPageClear(gstate);			   /* clear background */
	PFColourGray(gstate, DisplayPaper ? 0 : pt_hundred );

	/* select the system-default-font at the wanted (configured) size */
	PFFontLoad(gstate,DisplayFont);
	PFFontScale(gstate,g->fontsize);

	/* right justify the string ! */
	PFStringWidth(gstate, g->disp_txt, &width);
	PFMoveTo(gstate, (rinfo->xsiz - width), g->fontsize);

	/* draw the contents (just some text) */
	PFStringShow(gstate,g->disp_txt);

	return ERR_OK;
}




/***************
 *
 *		NumericKeyPressed
 * is called when .. a numeric key is pressed (briljant !) and updates the
 * info-string and the global value.
 *
 */

Error NumericKeyPressed(PWObject obj)
{
	Globals *g;
	char *txt;
	double fl;
	int digit, tmp, prec;
	char dummy[MAX_STR_LEN+1];

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);
	PWQuery(obj, PW_LOOSE_TEXT, &txt);

	g->isClear=FALSE;
	fl = g->typedIn;
	STRFormatGet(txt,&tmp,"%d",&digit);

	switch(g->dMode) {
	case DM_Dec :
		if (g->containsPoint) {
			/* containsPoint also indicates the power of ten */
			fl += (double) digit / pow(10.0,(double) g->containsPoint);
			prec = g->containsPoint++;
		} else {
			fl = (fl * 10) + (double) digit;
			prec = 0;
		}

		STRFormatPutSafe(g->disp_txt, MAX_STR_LEN, "%.*f", prec, fl);
		break;

	case DM_Hex:
	case DM_Bin: {
		int factor = (g->dMode == DM_Bin) ? 2 : 16;
		/* no decimals are possible ! */
		if (fl) {
			fl = fl*factor + (double) digit;
			STRAppendSafe(txt, g->disp_txt, MAX_STR_LEN);
		} else {
			fl = digit;
			STRCopySafe(txt, g->disp_txt, MAX_STR_LEN);
		}
		} break;
	}

	/* store the temporary result ! */
	g->typedIn = fl;

	PWChange(g->display,PW_CANVAS_REDRAW, NULL);

	return ERR_OK;
}


/***************
 *
 *		PointPressed
 * is called when . is pressed (briljant !) and updates the
 * the global data.
 *
 */

Error PointPressed(PWObject obj)
{
	Globals *g;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);
	g->isClear=FALSE;

	/* only allowable if dMode == DM_Dec !! */
	if (g->dMode == DM_Dec) {
		/* the point has been pressed, and the next number has to be
		 * added *10^-1 */
		g->containsPoint = 1;
	}

	return ERR_OK;
}


/***************
 *
 *		AlfaKeyPressed
 * is called when A..F is pressed (briljant !) and updates the
 * the global data.
 * NOTE : this can only be called when we are in HEX-modus.
 *
 */

Error AlfaKeyPressed(PWObject obj)
{
	Globals *g;
	char *txt;
	double fl;
	int digit, tmp, prec;
	char dummy[MAX_STR_LEN+1];

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);
	PWQuery(obj, PW_LOOSE_TEXT, &txt);

	g->isClear=FALSE;
	fl = g->typedIn;
	digit = ((int) txt[0] - (int) 'A') + 10;

	/* no decimals are possible ! */
	if (fl) {
		fl = fl * 16 + (double) digit;
		STRAppendSafe(txt, g->disp_txt, MAX_STR_LEN);
	} else {
		fl = digit;
		STRCopySafe(txt, g->disp_txt, MAX_STR_LEN);
	}
	

	/* store the temporary result ! */
	g->typedIn = fl;

	PWChange(g->display,PW_CANVAS_REDRAW, NULL);
	
	return ERR_OK;
}



/***************
 *
 *		ClearPressed
 * is called when ESC is pressed (briljant !) and updates the
 * the global data.
 *
 */

Error ClearPressed(PWObject obj)
{
	Globals *g;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);

	g->isClear=TRUE;
	g->containsPoint = 0;
	g->usedOp = Op_none;
	g->tmpRslt = g->typedIn = 0;
	STRCopySafe("0",g->disp_txt,MAX_STR_LEN);
	return PWChange(g->display,PW_CANVAS_REDRAW, NULL);
}

Error ESCPressed(PWObject obj)
{
	Globals *g;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);

	if (g->isClear)
		PWChange(obj,PW_SYSTEM_BREAKDOWN,NULL);
	else
	{
		g->isClear=TRUE;
		g->containsPoint = 0;
		g->usedOp = Op_none;
		g->tmpRslt = g->typedIn = 0;
		STRCopySafe("0",g->disp_txt,MAX_STR_LEN);
		PWChange(g->display,PW_CANVAS_REDRAW, NULL);
	}
	return ERR_OK;
}



/***************
 *
 *		Calculate (*to, *with, Operator)
 * executes
 *			*to  = *to OP *with
 *
 */

Error Calculate(double *to, double with, Operator_t op)
{

	switch(op) {
	case Op_mult:
		*to = *to * with;
		break;
	case Op_div:
		*to = *to / with;
		break;
	case Op_plus:
		*to = *to + with;
		break;
	case Op_min:
		*to = *to - with;
		break;
	}

	return ERR_OK;
}


/***************
 *
 *		ShowResult (txt, number, dMode)
 * fills the wanted string-representation of "number" in "txt";
 * -> depends on the current display-mode !
 * This is a more common showing routine than when a key is pressed !
 *
 */

Error ShowResult(char *txt, double number, DisplayMode_t dMode)
{

	int factor, nr, i;
	char zero_txt[MAX_STR_LEN+1];

	switch(dMode) {
	case DM_Hex:

		/* convert the displayed number */
		STRFormatPutSafe(txt,MAX_STR_LEN,"%X", (int) number);
		
		break;

	case DM_Dec:

		STRFormatPutSafe(txt,MAX_STR_LEN,"%.8f", number);

		/* we still need to remove the following zeroes (and possibly the
		 * point too) */
		for (i=strlen(txt)-1; i > 0; i--) {
			if (txt[i] != '0') {
				if (txt[i] == '.') {
					/* still skip this character ! */
					i--;
				}
				/* stop the for-loop */
				break;
			}
		}
		/* end the text where we ended ! */
		txt[i+1] = '\0';

		break;

	case DM_Bin:
		/* convert the displayed number */
		factor = 1;
		nr = (int) number;

		zero_txt[MAX_STR_LEN] = '\0';

		for (i=MAX_STR_LEN; i > 0; i--) {
			if (nr & factor) {
				zero_txt[i-1] = '1';
			} else {
				zero_txt[i-1] = '0';
			}
			factor = factor << 1;
		}

		/* copy the zero_txt to g->disp_txt (except the leading zeroes ) !! */
		for (i=0 ; zero_txt[i]=='0' ; i++);
		STRCopy(zero_txt+i,txt);
		break;
	}
}




/***************
 *
 *		OperatorKeyPressed
 * is called when .. a numeric key is pressed (briljant !) and updates the
 * info-string and the global value.
 *
 */

Error OperatorKeyPressed(PWObject obj)
{
	Globals *g;
	char *txt;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);
	PWQuery(obj, PW_LOOSE_TEXT, &txt);

	/* store temporary result ! */
	if (g->usedOp == Op_none)
	{
		/* nothing to do, just store it ! */
		g->tmpRslt = g->typedIn;
	} else if (g->usedOp == Op_equal) {
		/* even less to do --> we just work with the previously stored
		 * result !! */

	} else {
		/* calculate the temp answer ! */
		/* ... ??
		 */
		Calculate(&g->tmpRslt, g->typedIn, g->usedOp);

		ShowResult(g->disp_txt, g->tmpRslt, g->dMode);

		/* force a redraw: show the new temporary result */
		PWChange(g->display, PW_CANVAS_REDRAW, NULL);
	}

	/* store the pressed operator-key ! */
	switch(txt[0]) {
	case '+':
		g->usedOp = Op_plus;
		break;
	case '-':
		g->usedOp = Op_min;
		break;
	case '/':
		g->usedOp = Op_div;
		break;
	case '*':
		g->usedOp = Op_mult;
		break;
	}

	/* make sure that the next number will restart */
	g->typedIn = 0;
	g->containsPoint = 0;

	return ERR_OK;
}



/***************
 *
 *		EqualPressed
 * is called when = is pressed and shows the calculated result, and also
 * adjusts the global data.
 *
 */

Error EqualPressed(PWObject obj)
{
	Globals *g;

	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);

	Calculate(&g->tmpRslt, g->typedIn, g->usedOp);

	ShowResult(g->disp_txt, g->tmpRslt, g->dMode);

	g->typedIn = 0;
	g->containsPoint = 0;
	g->usedOp = Op_equal;

	/* force a redraw: show the new temporary result */
	PWChange(g->display, PW_CANVAS_REDRAW, NULL);

	return ERR_OK;
}

/***************
 *
 *		ModeKeyPressed
 * is called when any of the bin,dec or hex keys are pressed and changes to
 * the requested modus. The display needs to be adjusted for it : some keys
 * will no longer be available, and some will become so.
 *
 */

Error ModeKeyPressed(PWObject obj)
{
	Globals *g;
	DisplayMode_t prevMode;
	PWObject *desel_obj;
	char *txt;
	double number;
	
	PWQuery(obj, PW_GLOBAL_AUXILIARY, &g);

	prevMode = g->dMode;

	/** deselect the previous item **/
	switch (g->dMode) {
	case DM_Bin:
		desel_obj = &g->key_bin;
		break;
	case DM_Hex:
		desel_obj = &g->key_hex;
		break;
	case DM_Dec:
		desel_obj = &g->key_dec;
		break;
	}

	PWChange(*desel_obj,
			 PW_LOOSE_STATUS, (*desel_obj!=obj) ? PW_STATUS_AVAILABLE : PW_STATUS_SELECTED,
			 NULL);

	/* now get the selected mode !! */
	
	PWQuery(obj, PW_LOOSE_TEXT, &txt);

	if (txt==ItemHex)
	{
		/******* HEXADECIMAL MODE ********/
		/* make the Alfa-keys available */
		PWChange(g->key_a, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		PWChange(g->key_b, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		PWChange(g->key_c, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		PWChange(g->key_d, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		PWChange(g->key_e, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		PWChange(g->key_f, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		/* make the numeric keys available ! */
		if (prevMode == DM_Bin) {
			PWChange(g->key_2, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_3, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_4, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_5, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_6, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_7, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_8, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_9, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		}
		/* and make the '.' unavailable ! */
		PWChange(g->key_pt, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);

		/* and store the Mode */
		g->dMode = DM_Hex;

		/* convert to int ! */
		g->typedIn = (int) g->typedIn;
	}
	else
	if (txt==ItemDec)
	{
		/******* DECIMAL MODE ********/
		/* make the Alfa-keys unavailable */
		if (prevMode == DM_Hex) {
			PWChange(g->key_a, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_b, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_c, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_d, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_e, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_f, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		}
		/* make the numeric keys available ! */
		if (prevMode == DM_Bin) {
			PWChange(g->key_2, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_3, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_4, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_5, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_6, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_7, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_8, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
			PWChange(g->key_9, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);
		}

		/* and make the '.' available ! */
		PWChange(g->key_pt, PW_LOOSE_STATUS, PW_STATUS_AVAILABLE, NULL);

		/* and store the Mode */
		g->dMode = DM_Dec;

		g->containsPoint = 0;
	}
	else
	if (txt==ItemBin)
	{
		/******* BINARY MODE ********/
		if (prevMode == DM_Hex) {
			PWChange(g->key_a, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_b, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_c, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_d, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_e, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			PWChange(g->key_f, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
			
		}
		/* make the numeric keys unavailable ! (except 0-1) */
		PWChange(g->key_2, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_3, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_4, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_5, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_6, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_7, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_8, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);
		PWChange(g->key_9, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);

		/* and make the '.' unavailable ! */
		PWChange(g->key_pt, PW_LOOSE_STATUS, PW_STATUS_UNAVAILABLE, NULL);

		g->typedIn = (int) g->typedIn;

		/* and store the Mode */
		g->dMode = DM_Bin;
	}

	/* show the previous result converted to the current mode !
	 * This previous result either is what is already typed in, or
	 * the latest intermediary result !
	 */
	ShowResult(g->disp_txt, g->typedIn == 0.0 ? g->tmpRslt : g->typedIn,
			g->dMode);

	/* force a redraw: show the new temporary result */
	PWChange(g->display, PW_CANVAS_REDRAW, NULL);

	return ERR_OK;
}
